@@ -209,6 +209,12 @@ gp_filesystem_get_exif_mtime (CameraFilesystem *fs, const char *folder,
 	if (!fs)
 		return 0;
 
+	/* This is only useful for JPEGs. Avoid querying it for other types. */
+	if (	!strstr(filename,"jpg")  && !strstr(filename,"JPG") &&
+		!strstr(filename,"jpeg") && !strstr(filename,"JPEG")
+	)
+		return 0;
+
 	gp_file_new (&file);
 	if (gp_filesystem_get_file (fs, folder, filename,
 				GP_FILE_TYPE_EXIF, file, NULL) != GP_OK) {
@@ -308,7 +314,7 @@ delete_all_files (CameraFilesystem *fs, CameraFilesystemFolder *folder)
 	CameraFilesystemFile	*file;
 
 	CHECK_NULL (folder);
-	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Delete all files in folder %p", folder);
+	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Delete all files in folder %p/%s", folder, folder->name);
 
 	file = folder->files;
 	while (file) {
@@ -354,7 +360,7 @@ delete_folder (CameraFilesystem *fs, CameraFilesystemFolder **folder)
 	CameraFilesystemFolder *next;
 	CHECK_NULL (folder);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Delete one folder %p", *folder);
+	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Delete one folder %p/%s", *folder, (*folder)->name);
 	next = (*folder)->next;
 	delete_all_files (fs, *folder);
 	free ((*folder)->name);
@@ -441,6 +447,24 @@ lookup_folder_file (
 	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Lookup folder %s file %s", folder, filename);
 	xf = lookup_folder (fs, fs->rootfolder, folder, context);
 	if (!xf) return GP_ERROR_DIRECTORY_NOT_FOUND;
+	/* Check if we need to load the filelist of the folder ... */
+	if (xf->files_dirty) {
+		CameraList	*list;
+		int		ret;
+		/*
+                 * The folder is dirty. List the files in it to make it clean.
+		 */
+		gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Folder %s is dirty. "
+			"Listing files in there to make folder clean...", folder);
+		ret = gp_list_new (&list);
+		if (ret == GP_OK) {
+			ret = gp_filesystem_list_files (fs, folder, list, context);
+			gp_list_free (list);
+			gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Done making folder %s clean...", folder);
+		}
+		if (ret != GP_OK)
+			gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Making folder %s clean failed: %d", folder, ret);
+	}
 
 	f = xf->files;
 	while (f) {
@@ -459,7 +483,7 @@ static int
 recurse_delete_folder (CameraFilesystem *fs, CameraFilesystemFolder *folder) {
 	CameraFilesystemFolder  **f;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Recurse delete folder %p", folder);
+	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Recurse delete folder %p/%s", folder, folder->name);
 	f = &folder->folders;
 	while (*f) {
 		recurse_delete_folder (fs, *f);
@@ -516,7 +540,7 @@ append_to_folder (CameraFilesystemFolder *folder,
 	CameraFilesystemFolder	*f;
 	char	*s;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Append to folder %p - %s", folder, foldername);
+	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Append to folder %p/%s - %s", folder, folder->name, foldername);
 	/* Handle multiple slashes, and slashes at the end */
 	while (foldername[0]=='/')
 		foldername++;
@@ -697,7 +721,7 @@ int
 gp_filesystem_append (CameraFilesystem *fs, const char *folder,
 		      const char *filename, GPContext *context)
 {
-	CameraFilesystemFile *new;
+	CameraFilesystemFile **new;
 	CameraFilesystemFolder *f;
 
 	CHECK_NULL (fs && folder);
@@ -710,28 +734,26 @@ gp_filesystem_append (CameraFilesystem *fs, const char *folder,
 	if (!f)
 		CR (append_folder (fs, folder, &f, context));
 
-	new = f->files;
-	while (new) {
-		if (!strcmp(new->name, filename)) break;
-		new = new->next;
+	new = &f->files;
+	while (*new) {
+		if (!strcmp((*new)->name, filename)) break;
+		new = &((*new)->next);
 	}
-	if (new) {
+	if (*new) {
 		gp_context_error (context,
 			_("Could not append '%s' to folder '%s' because "
 			  "this file already exists."), filename, folder);
 		return (GP_ERROR_FILE_EXISTS);
 	}
 
-	CHECK_MEM (new = calloc (sizeof (CameraFilesystemFile), 1))
-	new->name = strdup (filename);
-	if (!new->name) {
-		free (new);
+	CHECK_MEM ((*new) = calloc (sizeof (CameraFilesystemFile), 1))
+	(*new)->name = strdup (filename);
+	if (!(*new)->name) {
+		free (*new);
+		*new = NULL;
 		return (GP_ERROR_NO_MEMORY);
 	}
-	new->info_dirty = 1;
-
-	new->next = f->files;
-	f->files = new;
+	(*new)->info_dirty = 1;
 	f->files_dirty = 0;
 	return (GP_OK);
 }
@@ -739,8 +761,16 @@ gp_filesystem_append (CameraFilesystem *fs, const char *folder,
 static void
 recursive_fs_dump (CameraFilesystemFolder *folder, int depth) {
 	CameraFilesystemFolder	*f;
+	CameraFilesystemFile	*xfile;
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-filesys", "%*sFolder %s", depth, " ", folder->name);
+
+	xfile = folder->files;
+	while (xfile) {
+		gp_log (GP_LOG_DEBUG, "gphoto2-filesys", "%*s    %s", depth, " ", xfile->name);
+		xfile = xfile->next;
+	}
+	
 	f = folder->folders;
 	while (f) {
 		recursive_fs_dump (f, depth+4);
@@ -1187,6 +1217,23 @@ gp_filesystem_remove_dir (CameraFilesystem *fs, const char *folder,
 	 */
 	f = lookup_folder (fs, fs->rootfolder, folder, context);
 	if (!f) return (GP_ERROR_DIRECTORY_NOT_FOUND);
+	/* Check if we need to load the folder ... */
+	if (f->folders_dirty) {
+		CameraList	*list;
+		int		ret;
+		/*
+		 * The owning folder is dirty. List the folders in it 
+		 * to make it clean.
+		 */
+		gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Folder %s is dirty. "
+			"Listing folders in there to make folder clean...", folder);
+		ret = gp_list_new (&list);
+		if (ret == GP_OK) {
+			ret = gp_filesystem_list_folders (fs, folder, list, context);
+			gp_list_free (list);
+			gp_log (GP_LOG_DEBUG, "gphoto2-filesystem", "Done making folder %s clean...", folder);
+		}
+	}
 	prev = &(f->folders);
 	while (*prev) {
 		if (!strcmp (name, (*prev)->name))
@@ -1628,8 +1675,11 @@ gp_filesystem_get_file_impl (CameraFilesystem *fs, const char *folder,
 	CR (gp_file_set_type (file, type));
 	CR (gp_file_set_name (file, filename));
 
+#if 0
+	/* this disables LRU completely. */
 	/* Cache this file */
 	CR (gp_filesystem_set_file_noop (fs, folder, file, context));
+#endif
 
 	/*
 	 * Often, thumbnails are of a different mime type than the normal
@@ -2300,6 +2350,7 @@ gp_filesystem_set_file_noop (CameraFilesystem *fs, const char *folder,
 		GP_DEBUG ("Searching data for mtime...");
 		CR (gp_file_get_data_and_size (file, NULL, &size));
 		if (size < 32*1024*1024) { /* just assume stuff above 32MB is not EXIF capable */
+			/* FIXME: Will leak on "fd" retrieval */
 			CR (gp_file_get_data_and_size (file, &data, &size));
 			t = get_exif_mtime ((unsigned char*)data, size);
 		}
